# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: RequestsCommon.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14RequestsCommon.proto\x12)com.igeolise.traveltime.rabbitmq.requests\"\"\n\x06\x43oords\x12\x0b\n\x03lat\x18\x01 \x01(\x02\x12\x0b\n\x03lng\x18\x02 \x01(\x02\"\xc6\x02\n\x0eTransportation\x12K\n\x04type\x18\x01 \x01(\x0e\x32=.com.igeolise.traveltime.rabbitmq.requests.TransportationType\x12\\\n\x0fpublicTransport\x18\x02 \x01(\x0b\x32\x41.com.igeolise.traveltime.rabbitmq.requests.PublicTransportDetailsH\x00\x12p\n\x19\x64rivingAndPublicTransport\x18\x03 \x01(\x0b\x32K.com.igeolise.traveltime.rabbitmq.requests.DrivingAndPublicTransportDetailsH\x00\x42\x17\n\x15transportationDetails\"6\n\x16PublicTransportDetails\x12\x1c\n\x14walkingTimeToStation\x18\x01 \x01(\r\"s\n DrivingAndPublicTransportDetails\x12\x1c\n\x14walkingTimeToStation\x18\x01 \x01(\r\x12\x1c\n\x14\x64rivingTimeToStation\x18\x02 \x01(\r\x12\x13\n\x0bparkingTime\x18\x03 \x01(\x11*\xb8\x01\n\x12TransportationType\x12\x14\n\x10PUBLIC_TRANSPORT\x10\x00\x12\x0b\n\x07\x44RIVING\x10\x01\x12 \n\x1c\x44RIVING_AND_PUBLIC_TRANSPORT\x10\x02\x12\x15\n\x11\x44RIVING_AND_FERRY\x10\x03\x12\x0b\n\x07WALKING\x10\x04\x12\x15\n\x11WALKING_AND_FERRY\x10\x07\x12\x0b\n\x07\x43YCLING\x10\x05\x12\x15\n\x11\x43YCLING_AND_FERRY\x10\x06*!\n\nTimePeriod\x12\x13\n\x0fWEEKDAY_MORNING\x10\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'RequestsCommon_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TRANSPORTATIONTYPE._serialized_start=606
  _TRANSPORTATIONTYPE._serialized_end=790
  _TIMEPERIOD._serialized_start=792
  _TIMEPERIOD._serialized_end=825
  _COORDS._serialized_start=67
  _COORDS._serialized_end=101
  _TRANSPORTATION._serialized_start=104
  _TRANSPORTATION._serialized_end=430
  _PUBLICTRANSPORTDETAILS._serialized_start=432
  _PUBLICTRANSPORTDETAILS._serialized_end=486
  _DRIVINGANDPUBLICTRANSPORTDETAILS._serialized_start=488
  _DRIVINGANDPUBLICTRANSPORTDETAILS._serialized_end=603
# @@protoc_insertion_point(module_scope)
